@@ -1,11 +1,14 @@
 #![cfg(ironman)]
 
 use crate::utils;
-use eu4save::models::{GameDifficulty, ProvinceEvent, ProvinceEventValue, TaxManpowerModifier};
 use eu4save::{
-    query::{LedgerPoint, NationEvent, NationEventKind, NationEvents, PlayerHistory, Query},
+    models::{GameDifficulty, ProvinceEvent, ProvinceEventValue, TaxManpowerModifier},
+    query::{
+        CountryManaUsage, LedgerPoint, NationEvent, NationEventKind, NationEvents, PlayerHistory,
+        Query,
+    },
     CountryTag, Encoding, Eu4Date, Eu4Extractor, Eu4ExtractorBuilder, FailedResolveStrategy,
-    ProvinceId,
+    PdsDate, ProvinceId,
 };
 use paste::paste;
 use std::io::Cursor;
@@ -36,7 +39,7 @@ fn test_eu4_bin() {
             latest: "CRO".parse().unwrap(),
             stored: "CRO".parse().unwrap(),
             events: vec![NationEvent {
-                date: Eu4Date::new(1769, 1, 2).unwrap(),
+                date: Eu4Date::from_ymd(1769, 1, 2),
                 kind: NationEventKind::TagSwitch("CRO".parse().unwrap()),
             }],
         },
@@ -58,12 +61,7 @@ fn test_eu4_kandy_bin() {
     let nation_events = query.nation_events(&province_owners);
     let histories = query.player_histories(&nation_events);
 
-    let player = query
-        .save()
-        .game
-        .countries
-        .get(&"BHA".parse().unwrap())
-        .unwrap();
+    let player = query.country(&"BHA".parse().unwrap()).unwrap();
     assert!(!player.completed_missions.is_empty());
 
     assert_eq!(
@@ -104,16 +102,7 @@ fn test_eu4_kandy_bin() {
         "C23".parse().unwrap(),
     ];
 
-    assert_eq!(
-        query
-            .save()
-            .game
-            .countries
-            .get(&"BHA".parse().unwrap())
-            .unwrap()
-            .subjects,
-        subjects
-    );
+    assert_eq!(player.subjects, subjects);
 
     // Testing binary encoded saves can extract province building history perfectly fine
     let london = query
@@ -135,13 +124,13 @@ fn test_eu4_kandy_bin() {
         .find(|(_date, key, _value)| key.as_str() == "marketplace")
         .unwrap();
     assert!(matches!(val, ProvinceEventValue::Bool(v) if v == &true));
-    assert_eq!(date.game_fmt().as_str(), "1486.6.3");
+    assert_eq!(date.game_fmt().to_string(), String::from("1486.6.3"));
 
     let building_date = query
         .province_building_history(london)
         .iter()
         .find(|x| x.building == "marketplace")
-        .map(|x| x.date.game_fmt());
+        .map(|x| x.date.game_fmt().to_string());
     assert_eq!(building_date, Some(String::from("1486.6.3")));
 
     let building_date = query
@@ -207,6 +196,24 @@ fn test_eu4_ita1() {
     );
 }
 
+#[test]
+fn test_inheritance_values() {
+    let data = utils::request("patch132.eu4");
+    let (save, _) = Eu4Extractor::extract_save(Cursor::new(&data[..])).unwrap();
+    let query = Query::from_save(save);
+
+    let inherit = query.inherit(&query.save_country(&"WUR".parse().unwrap()).unwrap());
+
+    assert_eq!(inherit.subtotal, 40672);
+    assert_eq!(inherit.inheritance_value, 16);
+    assert_eq!(inherit.start_t0_year, 1428);
+    assert_eq!(inherit.end_t0_year, 1502);
+    assert_eq!(inherit.start_t1_year, 1503);
+    assert_eq!(inherit.end_t1_year, 1507);
+    assert_eq!(inherit.start_t2_year, 1508);
+    assert_eq!(inherit.end_t2_year, 1527);
+}
+
 #[test]
 fn test_roundtrip_melt() {
     let data = utils::request("kandy2.bin.eu4");
@@ -295,7 +302,7 @@ ironman_test!(
         starting: "KND",
         player: "BHA",
         patch: "1.29.5.0",
-        date: Eu4Date::parse_from_str("1804.12.09").unwrap()
+        date: Eu4Date::parse("1804.12.09").unwrap()
     }
 );
 
@@ -306,7 +313,7 @@ ironman_test!(
         starting: "TYR",
         player: "TYR",
         patch: "1.30.3.0",
-        date: Eu4Date::parse_from_str("1581.03.01").unwrap()
+        date: Eu4Date::parse("1581.03.01").unwrap()
     }
 );
 
@@ -317,7 +324,7 @@ ironman_test!(
         starting: "MOS",
         player: "MOS",
         patch: "1.30.4.0",
-        date: Eu4Date::parse_from_str("1446.03.16").unwrap()
+        date: Eu4Date::parse("1446.03.16").unwrap()
     }
 );
 
@@ -329,11 +336,11 @@ fn trycone_expected_histories() -> Vec<PlayerHistory> {
             stored: "GBR".parse().unwrap(),
             events: vec![
                 NationEvent {
-                    date: Eu4Date::new(1518, 1, 29).unwrap(),
+                    date: Eu4Date::from_ymd(1518, 1, 29),
                     kind: NationEventKind::TagSwitch("IRE".parse().unwrap()),
                 },
                 NationEvent {
-                    date: Eu4Date::new(1606, 8, 4).unwrap(),
+                    date: Eu4Date::from_ymd(1606, 8, 4),
                     kind: NationEventKind::TagSwitch("GBR".parse().unwrap()),
                 },
             ],
@@ -350,19 +357,19 @@ fn leinster_history() -> NationEvents {
         stored: "LEI".parse().unwrap(),
         events: vec![
             NationEvent {
-                date: Eu4Date::new(1451, 8, 2).unwrap(),
+                date: Eu4Date::from_ymd(1451, 8, 2),
                 kind: NationEventKind::Annexed,
             },
             NationEvent {
-                date: Eu4Date::new(1588, 6, 15).unwrap(),
+                date: Eu4Date::from_ymd(1588, 6, 15),
                 kind: NationEventKind::Appeared,
             },
             NationEvent {
-                date: Eu4Date::new(1605, 2, 15).unwrap(),
+                date: Eu4Date::from_ymd(1605, 2, 15),
                 kind: NationEventKind::Annexed,
             },
             NationEvent {
-                date: Eu4Date::new(1716, 2, 9).unwrap(),
+                date: Eu4Date::from_ymd(1716, 2, 9),
                 kind: NationEventKind::Appeared,
             },
         ],
@@ -376,7 +383,7 @@ ironman_test!(
         starting: "TYR",
         player: "GBR",
         patch: "1.30.4.0",
-        date: Eu4Date::parse_from_str("1725.05.12").unwrap()
+        date: Eu4Date::parse("1725.05.12").unwrap()
     },
     |query: Query, _melted_data: &[u8]| {
         let province_owners = query.province_owners();
@@ -388,7 +395,7 @@ ironman_test!(
         assert_eq!(&histories, &trycone_expected_histories());
         let tag_resolver = query.tag_resolver(&nation_events);
         assert_eq!(
-            tag_resolver.resolve("IRE".parse().unwrap(), Eu4Date::new(1529, 3, 1).unwrap()),
+            tag_resolver.resolve("IRE".parse().unwrap(), Eu4Date::from_ymd(1529, 3, 1)),
             "GBR".parse().unwrap()
         );
 
@@ -612,11 +619,11 @@ fn true_heir_expected_histories() -> Vec<PlayerHistory> {
             stored: "MUG".parse().unwrap(),
             events: vec![
                 NationEvent {
-                    date: Eu4Date::new(1467, 12, 3).unwrap(),
+                    date: Eu4Date::from_ymd(1467, 12, 3),
                     kind: NationEventKind::TagSwitch("DLH".parse().unwrap()),
                 },
                 NationEvent {
-                    date: Eu4Date::new(1467, 12, 3).unwrap(),
+                    date: Eu4Date::from_ymd(1467, 12, 3),
                     kind: NationEventKind::TagSwitch("MUG".parse().unwrap()),
                 },
             ],
@@ -633,7 +640,7 @@ ironman_test!(
         starting: "SIS",
         player: "MUG",
         patch: "1.30.3.0",
-        date: Eu4Date::parse_from_str("1508.04.27").unwrap()
+        date: Eu4Date::parse("1508.04.27").unwrap()
     },
     |query: Query, _melted_data: &[u8]| {
         let province_owners = query.province_owners();
@@ -650,7 +657,7 @@ ironman_test!(
         starting: "KOL",
         player: "KOL",
         patch: "1.30.3.0",
-        date: Eu4Date::parse_from_str("1821.01.03").unwrap()
+        date: Eu4Date::parse("1821.01.03").unwrap()
     }
 );
 
@@ -661,7 +668,7 @@ ironman_test!(
         starting: "MLO",
         player: "ITA",
         patch: "1.29.6.0",
-        date: Eu4Date::parse_from_str("1547.03.05").unwrap()
+        date: Eu4Date::parse("1547.03.05").unwrap()
     },
     |query: Query, _melted_data: &[u8]| {
         assert_eq!(
@@ -682,7 +689,7 @@ ironman_test!(
         starting: "BUR",
         player: "BUR",
         patch: "1.30.4.0",
-        date: Eu4Date::parse_from_str("1821.01.03").unwrap()
+        date: Eu4Date::parse("1821.01.03").unwrap()
     }
 );
 
@@ -693,7 +700,7 @@ ironman_test!(
         starting: "BYZ",
         player: "BYZ",
         patch: "1.30.4.0",
-        date: Eu4Date::parse_from_str("1513.05.25").unwrap()
+        date: Eu4Date::parse("1513.05.25").unwrap()
     }
 );
 
@@ -704,7 +711,7 @@ ironman_test!(
         starting: "SZO",
         player: "SZO",
         patch: "1.30.4.0",
-        date: Eu4Date::parse_from_str("1800.01.01").unwrap()
+        date: Eu4Date::parse("1800.01.01").unwrap()
     },
     |query: Query, _melted_data: &[u8]| {
         assert_eq!(
@@ -720,7 +727,7 @@ fn cilli_history() -> NationEvents {
         latest: "CRO".parse().unwrap(),
         stored: "CRO".parse().unwrap(),
         events: vec![NationEvent {
-            date: Eu4Date::new(1509, 6, 6).unwrap(),
+            date: Eu4Date::from_ymd(1509, 6, 6),
             kind: NationEventKind::TagSwitch("CRO".parse().unwrap()),
         }],
     }
@@ -733,7 +740,7 @@ ironman_test!(
         starting: "CLI",
         player: "CRO",
         patch: "1.30.1.0",
-        date: Eu4Date::parse_from_str("1509.06.10").unwrap()
+        date: Eu4Date::parse("1509.06.10").unwrap()
     },
     |query: Query, _melted_data: &[u8]| {
         let province_owners = query.province_owners();
@@ -881,7 +888,7 @@ ironman_test!(
         starting: "CAS",
         player: "CAS",
         patch: "1.30.6.0",
-        date: Eu4Date::parse_from_str("1505.11.25").unwrap()
+        date: Eu4Date::parse("1505.11.25").unwrap()
     },
     |query: Query, _melted_data: &[u8]| {
         assert!(!query.save().meta.is_ironman);
@@ -895,7 +902,7 @@ ironman_test!(
         starting: "ENG",
         player: "ENG",
         patch: "1.31.0.0",
-        date: Eu4Date::parse_from_str("1444.11.11").unwrap()
+        date: Eu4Date::parse("1444.11.11").unwrap()
     },
     |_query: Query, melted_data: &[u8]| {
         // Find inukshuk
@@ -911,6 +918,80 @@ ironman_test!(
         starting: "NOR",
         player: "NOR",
         patch: "1.31.4.0",
-        date: Eu4Date::parse_from_str("1676.6.7").unwrap()
+        date: Eu4Date::parse("1676.6.7").unwrap()
+    },
+    |query: Query, _melted_data: &[u8]| {
+        let mod1 = &query.save().meta.mods_enabled_names[0];
+        assert_eq!(mod1.filename.as_str(), "mod/ugc_2146396184.mod");
+        assert_eq!(mod1.name.as_str(), "Bigger Stellaris");
+    }
+);
+
+ironman_test!(
+    tur,
+    "tur.eu4",
+    IronmanQuery {
+        starting: "TUR",
+        player: "TUR",
+        patch: "1.31.5.2",
+        date: Eu4Date::parse("1555.11.2").unwrap()
+    },
+    |query: Query, _melted_data: &[u8]| {
+        let player = query.country(&"TUR".parse().unwrap()).unwrap();
+        let m = query.country_mana_breakdown(&player);
+
+        #[rustfmt::skip]
+        fn assertions(m: CountryManaUsage) {
+            assert_eq!(m.adm.buy_idea + m.dip.buy_idea + m.mil.buy_idea, 6458);
+            assert_eq!(m.adm.advance_tech + m.dip.advance_tech + m.mil.advance_tech, 12584);
+            assert_eq!(m.adm.boost_stab + m.dip.boost_stab + m.mil.boost_stab, 138);
+            assert_eq!(m.adm.buy_general + m.dip.buy_general + m.mil.buy_general, 0);
+            assert_eq!(m.adm.buy_admiral + m.dip.buy_admiral + m.mil.buy_admiral, 0);
+            assert_eq!(m.adm.buy_conq + m.dip.buy_conq + m.mil.buy_conq, 0);
+            assert_eq!(m.adm.buy_explorer + m.dip.buy_explorer + m.mil.buy_explorer, 0);
+            assert_eq!(m.adm.develop_prov + m.dip.develop_prov + m.mil.develop_prov, 1025);
+            assert_eq!(m.adm.force_march + m.dip.force_march + m.mil.force_march, 0);
+            assert_eq!(m.adm.assault + m.dip.assault + m.mil.assault, 65);
+            assert_eq!(m.adm.seize_colony + m.dip.seize_colony + m.mil.seize_colony, 0);
+            assert_eq!(m.adm.burn_colony + m.dip.burn_colony + m.mil.burn_colony, 0);
+            assert_eq!(m.adm.attack_natives + m.dip.attack_natives + m.mil.attack_natives, 0);
+            assert_eq!(m.adm.scorch_earth + m.dip.scorch_earth + m.mil.scorch_earth, 60);
+            assert_eq!(m.adm.demand_non_wargoal_prov + m.dip.demand_non_wargoal_prov + m.mil.demand_non_wargoal_prov, 7551);
+            assert_eq!(m.adm.reduce_inflation + m.dip.reduce_inflation + m.mil.reduce_inflation, 420);
+            assert_eq!(m.adm.move_capital + m.dip.move_capital + m.mil.move_capital, 577);
+            assert_eq!(m.adm.make_province_core + m.dip.make_province_core + m.mil.make_province_core, 13654);
+            assert_eq!(m.adm.replace_rival + m.dip.replace_rival + m.mil.replace_rival, 0);
+            assert_eq!(m.adm.change_gov + m.dip.change_gov + m.mil.change_gov, 0);
+            assert_eq!(m.adm.change_culture + m.dip.change_culture + m.mil.change_culture, 0);
+            assert_eq!(m.adm.harsh_treatment + m.dip.harsh_treatment + m.mil.harsh_treatment, 381);
+            assert_eq!(m.adm.reduce_we + m.dip.reduce_we + m.mil.reduce_we, 502);
+            assert_eq!(m.adm.boost_faction + m.dip.boost_faction + m.mil.boost_faction, 0);
+            assert_eq!(m.adm.raise_war_taxes + m.dip.raise_war_taxes + m.mil.raise_war_taxes, 0);
+            assert_eq!(m.adm.increse_tariffs + m.dip.increse_tariffs + m.mil.increse_tariffs, 0);
+            assert_eq!(m.adm.promote_merc + m.dip.promote_merc + m.mil.promote_merc, 0);
+            assert_eq!(m.adm.decrease_tariffs + m.dip.decrease_tariffs + m.mil.decrease_tariffs, 0);
+            assert_eq!(m.adm.move_trade_port + m.dip.move_trade_port + m.mil.move_trade_port, 190);
+            assert_eq!(m.adm.create_trade_post + m.dip.create_trade_post + m.mil.create_trade_post, 0);
+            assert_eq!(m.adm.siege_sorties + m.dip.siege_sorties + m.mil.siege_sorties, 45);
+            assert_eq!(m.adm.buy_religious_reform + m.dip.buy_religious_reform + m.mil.buy_religious_reform, 0);
+            assert_eq!(m.adm.set_primary_culture + m.dip.set_primary_culture + m.mil.set_primary_culture, 0);
+            assert_eq!(m.adm.add_accepted_culture + m.dip.add_accepted_culture + m.mil.add_accepted_culture, 93);
+            assert_eq!(m.adm.remove_accepted_culture + m.dip.remove_accepted_culture + m.mil.remove_accepted_culture, 0);
+            assert_eq!(m.adm.strengthen_government + m.dip.strengthen_government + m.mil.strengthen_government, 0);
+            assert_eq!(m.adm.boost_militarization + m.dip.boost_militarization + m.mil.boost_militarization, 0);
+            assert_eq!(m.adm.artillery_barrage + m.dip.artillery_barrage + m.mil.artillery_barrage, 862);
+            assert_eq!(m.adm.establish_siberian_frontier + m.dip.establish_siberian_frontier + m.mil.establish_siberian_frontier, 0);
+            assert_eq!(m.adm.government_interaction + m.dip.government_interaction + m.mil.government_interaction, 0);
+            assert_eq!(m.adm.naval_barrage + m.dip.naval_barrage + m.mil.naval_barrage, 381);
+            assert_eq!(m.adm.add_tribal_land + m.dip.add_tribal_land + m.mil.add_tribal_land, 0);
+            assert_eq!(m.adm.force_march + m.dip.force_march + m.mil.force_march, 0);
+            assert_eq!(m.adm.create_leader + m.dip.create_leader + m.mil.create_leader, 6977);
+            assert_eq!(m.adm.enforce_culture + m.dip.enforce_culture + m.mil.enforce_culture, 0);
+            assert_eq!(m.adm.effect + m.dip.effect + m.mil.effect, 0);
+            assert_eq!(m.adm.minority_expulsion + m.dip.minority_expulsion + m.mil.minority_expulsion, 0);
+            assert_eq!(m.adm.other + m.dip.other + m.mil.other, 0);
+        }
+
+        assertions(m)
     }
 );
